/* mapview - map viewer for PalmOS
   Copyright (C) 2005  Edwin Steiner <edwin.steiner@gmx.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <PalmOS.h>
#include "map.h"
#include "memory.h"
#include "debug.h"
#include <DataMgr.h>

typedef struct mapview_rc_map_t mapview_rc_map_t;
typedef struct mapview_rc_layer_t mapview_rc_layer_t;

struct mapview_rc_map_t {
        UInt32  mapsizex;
        UInt32  mapsizey;
        UInt16  nlayers;
        UInt16  density;
};

struct mapview_rc_layer_t {
        UInt16  scale;
        UInt16  tilesizex;
        UInt16  tilesizey;
        UInt16  tilesx;
        UInt16  tilesy;
        UInt16  tilebaseid;
};

static MemHandle
mapview_map_get_resource(mapview_map_t *map,DmResType type,DmResID id)
{
        UInt16 index;
        
        ASSERT(map);
        
        if (!map->dbref)
                return NULL;
        index = DmFindResource(map->dbref,type,id,NULL);

        return DmGetResourceIndex(map->dbref,index);
}

mapview_map_t *
mapview_map_new(UInt16 cardno,LocalID dbid)
{
        mapview_map_t *map;
        mapview_rc_map_t *maprc;
        MemHandle maprchandle;
        int i;

        map = NEW(mapview_map_t);
        if (!map)
                return NULL;

        map->dbref = DmOpenDatabase(cardno,dbid,dmModeReadOnly);
        if (!map->dbref)
                return NULL;

        maprchandle = mapview_map_get_resource(map,MapviewMapResourceType,MapviewMaprcID);
        if (!maprchandle)
                return NULL;

        maprc = (mapview_rc_map_t *) MemHandleLock(maprchandle);
        if (!maprc)
                return NULL;

        map->mapsizex  = maprc->mapsizex;
        map->mapsizey  = maprc->mapsizey;
        map->nlayers   = maprc->nlayers;
        map->density   = maprc->density;

        ASSERT(map->nlayers > 0);
        
        if (MemHandleUnlock(maprchandle) != errNone)
                return NULL;

        map->layers = NEWARRAY(mapview_layer_t*,map->nlayers);
        if (!map->layers)
                return NULL;

        for (i=0; i<map->nlayers; ++i) {
                map->layers[i] = mapview_layer_new(map,MapviewLayerrcID + i);
                if (!map->layers[i])
                        return NULL;
        }

        return map;
}

void
mapview_map_free(mapview_map_t *map)
{
        int i;
        
        if (!map)
                return;

#if 0
        if (DmReleaseResource(map->_indexhandle) != errNone)
                return;
#endif

        for (i=0; i<map->nlayers; ++i)
                mapview_layer_free(map->layers[i]);

        if (map->dbref)
                DmCloseDatabase(map->dbref);

        FREEARRAY(mapview_layer_t*,map->nlayers,map->layers);
        FREE(mapview_map_t,map);
}

mapview_layer_t *
mapview_layer_new(mapview_map_t *map,UInt32 id)
{
        mapview_layer_t *layer;
        mapview_rc_layer_t *layerrc;
        MemHandle rchandle;
        
        ASSERT(map);

        layer = NEW(mapview_layer_t);
        if (!layer)
                return NULL;

        layer->_map = map;

        rchandle = mapview_map_get_resource(map,MapviewLayerResourceType,id);
        if (!rchandle)
                return NULL;

        layerrc = (mapview_rc_layer_t *) MemHandleLock(rchandle);
        if (!layerrc)
                return NULL;

        layer->scale      = layerrc->scale;
        layer->tilesizex  = layerrc->tilesizex;
        layer->tilesizey  = layerrc->tilesizey;
        layer->tilesx     = layerrc->tilesx;
        layer->tilesy     = layerrc->tilesy;
        layer->tilebaseid = layerrc->tilebaseid;

        if (MemHandleUnlock(rchandle) != errNone)
                return NULL;

        layer->_bitmaphandle = NULL;

        return layer;
}

void
mapview_layer_free(mapview_layer_t *layer)
{
        if (!layer)
                return;

        if (layer->_bitmaphandle != NULL)
                MemHandleUnlock(layer->_bitmaphandle);

        FREE(mapview_layer_t,layer);
}

BitmapPtr 
mapview_layer_get_tile(mapview_layer_t *layer,Int32 tx,Int32 ty)
{
        UInt32 id;

        ASSERT(layer);
        ASSERT(layer->_bitmaphandle == NULL);

        if (tx < 0 || ty < 0 || tx >= layer->tilesx || ty >= layer->tilesy)
                return NULL;
        
        id = (UInt32)layer->tilebaseid + (ty*layer->tilesx + tx);

        layer->_bitmaphandle = mapview_map_get_resource(layer->_map,bitmapRsc,id);
        if (!layer->_bitmaphandle)
                return NULL;

        return (BitmapPtr) MemHandleLock(layer->_bitmaphandle);
}

void
mapview_layer_release_tile(mapview_layer_t *layer)
{
        ASSERT(layer);
        ASSERT(layer->_bitmaphandle != NULL);

        MemHandleUnlock(layer->_bitmaphandle);
        layer->_bitmaphandle = NULL;
}


/* vim: et sts=8 sw=8 ai
 */
